-- Copyright (c) 2012 Patrick Jones http://patrickmjones.com
-- 
-- Permission is hereby granted, free of charge, to any person obtaining
-- a copy of this software and associated documentation files (the
-- "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish,
-- distribute, sublicense, and/or sell copies of the Software, and to
-- permit persons to whom the Software is furnished to do so, subject to
-- the following conditions:
-- 
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
-- 
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
-- LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
-- OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
-- WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-- 
-- 
--
-- Controls:
--  Left Player: "a" moves paddle up, "z" moves paddle down
--  Right Player: "up" key moves paddle up, "down" key moves paddle down
--  "Spacebar" to serve the ball: 
--    NOTE: left player serves first and last scoring player serves thereafter
--
--
--
-- pjones-pong
function love.load()
	screenX = love.graphics.getWidth()
	screenY = love.graphics.getHeight()

	-- Basic configuration
	gameconfig = {}
	gameconfig.paddles = {}
	gameconfig.paddles.width = 15
	gameconfig.paddles.height = 70
	gameconfig.paddles.speed = 600
	gameconfig.ball = {}
	gameconfig.ball.vspeedmin = 50
	gameconfig.ball.vspeedmax = 600
	gameconfig.ball.speedinitial = 150
	gameconfig.ball.speedplus = 100

	-- p1 specific config
	paddle1 = initialize_paddle()
	paddle1.upKey = "a"
	paddle1.downKey = "z"

	-- p2 specific config
	paddle2 = initialize_paddle()
	paddle2.x = screenX - gameconfig.paddles.width
	paddle2.upKey = "up"
	paddle2.downKey = "down"

	-- ball initialization
	ball = {}
	ball.inmotion = false
	ball.direction = "l"
	ball.vdirection = "d"
	ball.speed = gameconfig.ball.speedinitial
	ball.vspeed = 0
	ball.height = 10
	ball.width = 10

	recenter_ball()

	-- load necessary sound effects
	sounds = {}
	sounds.blip = love.audio.newSource( "blip.wav", "static" )
	sounds.score = love.audio.newSource( "score.wav", "static" )
end

-- This is the main logic part of the loop
function love.update(dt)
	move_paddle(paddle1, dt)
	move_paddle(paddle2, dt)

	-- press "space" to serve
	if ball.inmotion == false and love.keyboard.isDown(" ") then
		ball.inmotion = true
		ball.vspeed = math.random(gameconfig.ball.vspeedmin, gameconfig.ball.vspeedmax)
	end
	if ball.inmotion == false then
		recenter_ball()
	end

	-- Move the ball as necessary
	if ball.inmotion == true then
		if ball.direction == "l" then
			ball.x = ball.x - ball.speed*dt			
		else
			ball.x = ball.x + ball.speed*dt
		end

		if ball.vdirection == "u" then
			ball.y = ball.y - ball.vspeed*dt
		else
			ball.y = ball.y + ball.vspeed*dt
		end
	end

	-- ball/paddle collision
	if ball.x < paddle1.x + paddle1.width and ball.y > paddle1.y and ball.y < paddle1.y + paddle1.height and not (ball.direction == "r") then
		paddle_collision()
	elseif ball.x > paddle2.x - 5 and ball.y > paddle2.y and ball.y < paddle2.y + paddle2.height and not (ball.direction == "l") then
		paddle_collision()
	end

	-- ball edge bouncing
	if ball.y < 0 then -- ball hit top of screen
		ball.vdirection = "d" 
	elseif ball.y > screenY - ball.height then -- ball hit bottom of screen
		ball.vdirection = "u"
	end

	-- See if anyone scored
	if ball.x < 0 then -- p2 scored!
		point_scored(paddle2) 
	elseif ball.x > screenX - ball.width then -- p1 scored!
		point_scored(paddle1)
	end
end

-- Now that all the calculations are done, we just draw everything on screen
function love.draw()
	love.graphics.print(paddle1.score, screenX * 0.25, 5)
	love.graphics.print(paddle2.score, screenX * 0.75, 5)
	love.graphics.setColor(255,255,255,255)
	love.graphics.rectangle("fill", paddle1.x, paddle1.y, paddle1.width, paddle1.height)
	love.graphics.rectangle("fill", paddle2.x, paddle2.y, paddle2.width, paddle2.height)
	love.graphics.rectangle("fill", ball.x, ball.y, ball.width, ball.height)
end

-- Sets the ball to the center of the screen
function recenter_ball() 
	ball.x = (screenX * 0.5) - (ball.width * 0.5)
	ball.y = (screenY * 0.5) - (ball.height * 0.5)
end

-- Basic paddle setup
function initialize_paddle()
	local paddle = {}
	paddle.x = 0
	paddle.y = 0
	paddle.speed = gameconfig.paddles.speed
	paddle.height = gameconfig.paddles.height
	paddle.width = gameconfig.paddles.width
	paddle.score = 0
	return paddle
end

-- Check if a paddle's configured key is pressed for up or down and moves it accordlingly
function move_paddle(paddle, dt)
	if love.keyboard.isDown(paddle.upKey) then
		paddle.y = paddle.y - paddle.speed*dt
	elseif love.keyboard.isDown(paddle.downKey) then
		paddle.y = paddle.y + paddle.speed*dt
	end

 	if paddle.y < 0 then
		paddle.y = 0
	elseif paddle.y > screenY - paddle.height then
		paddle.y = screenY - paddle.height
	end
end

-- Ball has collided with a paddle, take appropriate action
function paddle_collision()
	-- bounce the ball back, if "r"ight then go "l"eft and vice versa
	if ball.direction == "l" then
		ball.direction = "r"
	else
		ball.direction = "l"
	end

	-- increase ball speed each time it is hit
	ball.speed = ball.speed + gameconfig.ball.speedplus

	-- randomize the vertical bounce
	ball.vspeed = math.random(gameconfig.ball.vspeedmin, gameconfig.ball.vspeedmax)	

	-- play sound
	love.audio.rewind(sounds.blip)
	love.audio.play(sounds.blip)
end

function point_scored(paddle)
	ball.inmotion = false
	ball.speed = gameconfig.ball.speedinitial
	paddle.score = paddle.score + 1
	love.audio.rewind(sounds.score)
	love.audio.play(sounds.score)
end
